/* -*- C++ -*-
 * lemon/traits.h - Part of LEMON, a generic C++ optimization library
 *
 * Copyright (C) 2006 Egervary Jeno Kombinatorikus Optimalizalasi Kutatocsoport
 * (Egervary Research Group on Combinatorial Optimization, EGRES).
 *
 * Permission to use, modify and distribute this software is granted
 * provided that this copyright notice appears in all copies. For
 * precise terms see the accompanying LICENSE file.
 *
 * This software is provided "AS IS" with no warranty of any kind,
 * express or implied, and with no claim as to its suitability for any
 * purpose.
 *
 */

#ifndef LEMON_TRAITS_H
#define LEMON_TRAITS_H

#include <lemon/utility.h>

///\file
///\brief Traits for graphs and maps
///

namespace lemon {
  template <typename _Graph, typename _Item>
  class ItemSetTraits {};
  
  template <typename _Graph>
  class ItemSetTraits<_Graph, typename _Graph::Node> {
  public:
    
    typedef _Graph Graph;

    typedef typename Graph::Node Item;
    typedef typename Graph::NodeIt ItemIt;

    template <typename _Value>
    class Map : public Graph::template NodeMap<_Value> {
    public:
      typedef typename Graph::template NodeMap<_Value> Parent; 
      typedef typename Parent::Value Value;

      Map(const Graph& _graph) : Parent(_graph) {}
      Map(const Graph& _graph, const Value& _value) 
	: Parent(_graph, _value) {}
    };

  };

  template <typename _Graph>
  class ItemSetTraits<_Graph, typename _Graph::Edge> {
  public:
    
    typedef _Graph Graph;

    typedef typename Graph::Edge Item;
    typedef typename Graph::EdgeIt ItemIt;

    template <typename _Value>
    class Map : public Graph::template EdgeMap<_Value> {
    public:
      typedef typename Graph::template EdgeMap<_Value> Parent; 
      typedef typename Parent::Value Value;

      Map(const Graph& _graph) : Parent(_graph) {}
      Map(const Graph& _graph, const Value& _value) 
	: Parent(_graph, _value) {}
    };

  };

  template <typename _Graph>
  class ItemSetTraits<_Graph, typename _Graph::UndirEdge> {
  public:
    
    typedef _Graph Graph;

    typedef typename Graph::UndirEdge Item;
    typedef typename Graph::UndirEdgeIt ItemIt;

    template <typename _Value>
    class Map : public Graph::template UndirEdgeMap<_Value> {
    public:
      typedef typename Graph::template UndirEdgeMap<_Value> Parent; 
      typedef typename Parent::Value Value;

      Map(const Graph& _graph) : Parent(_graph) {}
      Map(const Graph& _graph, const Value& _value) 
	: Parent(_graph, _value) {}
    };

  };

  template <typename Map, typename Enable = void>
  struct MapTraits {
    typedef False ReferenceMapTag;

    typedef typename Map::Key Key;
    typedef typename Map::Value Value;

    typedef const Value ConstReturnValue;
    typedef const Value ReturnValue;
  };

  template <typename Map>
  struct MapTraits<
    Map, typename enable_if<typename Map::ReferenceMapTag, void>::type > 
  {
    typedef True ReferenceMapTag;
    
    typedef typename Map::Key Key;
    typedef typename Map::Value Value;

    typedef typename Map::ConstReference ConstReturnValue;
    typedef typename Map::Reference ReturnValue;

    typedef typename Map::ConstReference ConstReference; 
    typedef typename Map::Reference Reference;
 };

  // Indicators for the tags

  template <typename Graph, typename Enable = void>
  struct NodeNumTagIndicator {
    static const bool value = false;
  };

  template <typename Graph>
  struct NodeNumTagIndicator<
    Graph, 
    typename enable_if<typename Graph::NodeNumTag, void>::type
  > {
    static const bool value = true;
  };

  template <typename Graph, typename Enable = void>
  struct EdgeNumTagIndicator {
    static const bool value = false;
  };

  template <typename Graph>
  struct EdgeNumTagIndicator<
    Graph, 
    typename enable_if<typename Graph::EdgeNumTag, void>::type
  > {
    static const bool value = true;
  };

  template <typename Graph, typename Enable = void>
  struct FindEdgeTagIndicator {
    static const bool value = false;
  };

  template <typename Graph>
  struct FindEdgeTagIndicator<
    Graph, 
    typename enable_if<typename Graph::FindEdgeTag, void>::type
  > {
    static const bool value = true;
  };



}

#endif // LEMON_MAPS_H
